@@ -1,6 +1,6 @@
 /* Superoptimizer definitions.
 
-   Copyright (C) 1991, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -18,19 +18,22 @@
 
 #if !(defined(SPARC) || defined(POWER) || defined(POWERPC) || defined(M88000) \
       || defined(AM29K) || defined(MC68000) || defined(MC68020) \
-      || defined(I386) || defined(PYR) || defined(ALPHA) || defined(HPPA))
+      || defined(I386) || defined(PYR) || defined(ALPHA) || defined(HPPA) \
+      || defined(SH))
 /* If no target instruction set is defined, use host instruction set.  */
 #define SPARC (defined(sparc) || defined(__sparc__))
-#define POWER (defined(rs6000) || defined(_IBMR2))
+#define POWER ((defined(rs6000) || defined(_IBMR2)) && !defined (_ARCH_PPC))
+#define POWEPC (defined(_ARCH_PPC))
 /* #define POWER ???? */
 #define M88000 (defined(m88000) || defined(__m88000__))
 #define AM29K (defined(_AM29K) || defined(_AM29000))
 #define MC68020 (defined(m68020) || defined(mc68020))
 #define MC68000 (defined(m68000) || defined(mc68000))
-#define I386 (defined(i386) || defined(i80386))
+#define I386 (defined(i386) || defined(i80386) || defined(__i386__))
 #define PYR (defined(pyr) || defined(__pyr__))
 #define ALPHA defined(__alpha)
 #define HPPA defined(__hppa)
+#define SH defined(__sh__)
 #endif
 
 #define M68000 (MC68000 || MC68020)
@@ -38,8 +41,50 @@
 #define POWER 1
 #endif
 
+#if SPARC
+#define TARGET_STRING "SPARC v7/v8"
+#elif POWERPC
+#define TARGET_STRING "PowerPC"
+#elif POWER
+#define TARGET_STRING "IBM POWER"
+#elif M88000
+#define TARGET_STRING "Motorola MC88000"
+#elif AM29K
+#define TARGET_STRING "Amd 29000"
+#elif MC68020
+#define TARGET_STRING "Motorola MC68020"
+#elif MC68000
+#define TARGET_STRING "Motorola MC68000 (no 68020 instructions)"
+#elif I386
+#define TARGET_STRING "Intel 386/486/Pentium/Sexium"
+#elif PYR
+#define TARGET_STRING "Pyramid (with the secret instructions)"
+#elif ALPHA
+#define TARGET_STRING "DEC Alpha"
+#elif HPPA
+#define TARGET_STRING "Hewlett-Packard Precision Architecture (PA-RISC)"
+#elif  SH
+#define TARGET_STRING "Hitatchi Super-H (SH)"
+#define EXTRA_SEQUENCE_TESTS(seq, n) \
+{									\
+  int reg0 = -1;							\
+  int i;								\
+  for (i = 0; i < n; i++)						\
+    {									\
+      if ((seq[i].opcode == AND || seq[i].opcode == XOR			\
+	   || seq[i].opcode == IOR || seq[i].opcode == CYEQ)		\
+	  && IMMEDIATE_P (seq[i].s2))					\
+	{								\
+	  if (reg0 >= 0)						\
+	    return;							\
+	  reg0 = i;							\
+	}								\
+    }									\
+}
+#endif
+
 #if !(SPARC || POWER || M88000 || AM29K || M68000 || I386 || PYR \
-      || ALPHA || HPPA)
+      || ALPHA || HPPA || SH)
 #error You have to choose target CPU type (e.g. -DSPARC).
 #endif
 
@@ -49,8 +94,15 @@
 #define SHIFT_COST(CNT) 1
 #endif
 
+#ifdef SHIFTS
+#undef SHIFTS
+#define SHIFTS 1
+#else
+#define SHIFTS 0
+#endif
+
 #ifndef USE_ASM
-#define NO_ASM 1
+#define NO_ASM 1		/* Used by longlong.h */
 #endif
 
 #include "longlong.h"
@@ -96,12 +148,13 @@ typedef unsigned_word word;
 #endif
 
 /* Handle immediates by putting all handled values in the VALUE array at
-   appropriate indices, and then insert these indices in the code???
-   Interesting constants are probably 0, 1, -1, 0x80000000, and
-   0x7FFFFFFF.  */
+   appropriate indices, and then insert these indices in the code.
+   We currently do this just for some hardwired constants.  */
 
 #define CNST_0x80000000 (0x20 - 2)
 #define CNST_0x7FFFFFFF (0x20 - 3)
+#define CNST_0xFFFF (0x20 - 4)
+#define CNST_0xFF (0x20 - 5)
 
 #define VALUE_MIN_SIGNED ((word) 1 << (BITS_PER_WORD - 1))
 #define VALUE_MAX_SIGNED (((word) 1 << (BITS_PER_WORD - 1)) - 1)
@@ -111,11 +164,13 @@ typedef unsigned_word word;
 
 /* The IMMEDIATE_* macros are for printing assembler.  NOT for sequence
    generating or analyze.  */
-#define IMMEDIATE_P(op) (op >= 0x20 - 3)
+#define IMMEDIATE_P(op) (op >= 0x20 - 5)
 static const word __immediate_val[] =
 {
   VALUE_MIN_SIGNED,
   VALUE_MAX_SIGNED,
+  0xffff,
+  0xff
 };
 #define IMMEDIATE_VAL(op) \
   ((op) >= 0x20 - 1 ? op - 0x20 : __immediate_val[0x20 - 2 - (op)])
@@ -236,7 +291,7 @@ typedef struct
        : "r" (r1), "Or" (r2))
 #endif /* m88k */
 
-#endif /* __GNUC__ && USE_ASM*/
+#endif /* __GNUC__ && USE_ASM */
 
 /************************* Default PERFORM_* in C *************************/
 
@@ -482,27 +537,53 @@ typedef struct
        (co) = ((r1) >> TRUNC_CNT(BITS_PER_WORD - (r2))) & 1;		\
        (d) = __d; } while (0)
 #endif
+/* Do these first two rotates actually set carry correctly for r2 == 0??? */
 #ifndef PERFORM_ROTATEL_CO
 #define PERFORM_ROTATEL_CO(d, co, r1, r2, ci) \
-  ((d) = ((r1) << TRUNC_CNT(r2)) | ((r1) >> TRUNC_CNT(BITS_PER_WORD - (r2))),\
+  ((d) = (((r1) << TRUNC_CNT(r2))					\
+	  | ((unsigned_word) (r1) >> TRUNC_CNT(BITS_PER_WORD - (r2)))),	\
    (co) = (d) & 1)
 #endif
+#ifndef PERFORM_ROTATER_CO
+#define PERFORM_ROTATER_CO(d, co, r1, r2, ci) \
+  ((d) = (((r1) >> TRUNC_CNT(r2))					\
+	  | ((unsigned_word) (r1) << TRUNC_CNT(BITS_PER_WORD - (r2)))),	\
+   (co) = ((d) >> (BITS_PER_WORD - 1)) & 1)
+#endif
 #ifndef PERFORM_ROTATEXL_CIO
 #define PERFORM_ROTATEXL_CIO(d, co, r1, r2, ci) \
   do { word __d;  unsigned cnt = TRUNC_CNT(r2);				\
-       if (cnt == 1)							\
+       if (cnt != 0)							\
 	 {								\
-	   __d = ((r1) << 1) | (ci);					\
-	   (co) = (r1) >> (BITS_PER_WORD - 1);				\
+	   __d = ((r1) << cnt) | ((ci) << (cnt - 1));			\
+	   if (cnt != 1)						\
+	     __d |= (unsigned_word) (r1) >> (BITS_PER_WORD - (cnt - 1));\
+	   (co) = ((unsigned_word) (r1) >> (BITS_PER_WORD - cnt)) & 1;	\
+	   (d) = __d;							\
 	 }								\
        else								\
 	 {								\
-	   __d = ((r1) << cnt)						\
-	     | (ci) << (cnt - 1)					\
-	     | ((r1) >> (BITS_PER_WORD + 1 - cnt));			\
-	   (co) = ((r1) >> (BITS_PER_WORD - cnt)) & 1;			\
+	   (co) = (ci);							\
+	   (d) = (r1);							\
+	 }								\
+     } while (0)
+#endif
+#ifndef PERFORM_ROTATEXR_CIO
+#define PERFORM_ROTATEXR_CIO(d, co, r1, r2, ci) \
+  do { word __d;  unsigned cnt = TRUNC_CNT(r2);				\
+       if (cnt != 0)							\
+	 {								\
+	   __d = ((unsigned_word) (r1) >> cnt) | ((ci) << (BITS_PER_WORD - cnt)); \
+	   if (cnt != 1)						\
+	     __d |= ((r1) << (BITS_PER_WORD - (cnt - 1)));		\
+	   (co) = ((unsigned_word) (r1) >> (cnt - 1)) & 1;		\
+	   (d) = __d;							\
+	 }								\
+       else								\
+	 {								\
+	   (co) = (ci);							\
+	   (d) = (r1);							\
 	 }								\
-       (d) = __d;							\
      } while (0)
 #endif
 #ifndef PERFORM_EXTS1
@@ -513,6 +594,14 @@ typedef struct
 #define PERFORM_EXTS2(d, co, r1, r2, ci) \
   ((d) = ((signed_word) (r1) >> TRUNC_CNT(r2)) << 30 >> 30, (co) = (ci))
 #endif
+#ifndef PERFORM_EXTS8
+#define PERFORM_EXTS8(d, co, r1, r2, ci) \
+  ((d) = ((signed_word) (r1) >> TRUNC_CNT(r2)) << 24 >> 24, (co) = (ci))
+#endif
+#ifndef PERFORM_EXTS16
+#define PERFORM_EXTS16(d, co, r1, r2, ci) \
+  ((d) = ((signed_word) (r1) >> TRUNC_CNT(r2)) << 16 >> 16, (co) = (ci))
+#endif
 #ifndef PERFORM_EXTU1
 #define PERFORM_EXTU1(d, co, r1, r2, ci) \
   ((d) = ((unsigned_word) (r1) >> TRUNC_CNT(r2)) & 1, (co) = (ci))
@@ -590,6 +679,39 @@ typedef struct
   ((d) = (unsigned_word) (r1) < (unsigned_word) (r2), (co) = (ci))
 #endif
 
+#ifndef PERFORM_CYEQ
+#define PERFORM_CYEQ(d, co, r1, r2, ci) \
+  ((co) = (r1) == (r2))
+#endif
+#ifndef PERFORM_CYGES
+#define PERFORM_CYGES(d, co, r1, r2, ci) \
+  ((co) = (signed_word) (r1) >= (signed_word) (r2))
+#endif
+#ifndef PERFORM_CYGEU
+#define PERFORM_CYGEU(d, co, r1, r2, ci) \
+  ((co) = (unsigned_word) (r1) >= (unsigned_word) (r2))
+#endif
+#ifndef PERFORM_CYGTS
+#define PERFORM_CYGTS(d, co, r1, r2, ci) \
+  ((co) = (signed_word) (r1) > (signed_word) (r2))
+#endif
+#ifndef PERFORM_CYGTU
+#define PERFORM_CYGTU(d, co, r1, r2, ci) \
+  ((co) = (unsigned_word) (r1) > (unsigned_word) (r2))
+#endif
+#ifndef PERFORM_CYAND
+#define PERFORM_CYAND(d, co, r1, r2, ci) \
+  ((co) = ((r1) & (r2)) == 0)
+#endif
+#ifndef PERFORM_MERGE16
+#define PERFORM_MERGE16(d, co, r1, r2, ci) \
+  ((d) = ((word) (r1) >> 16) | ((r2) << 16), (co) = (ci))
+#endif
+#ifndef PERFORM_DECR_CYEQ
+#define PERFORM_DECR_CYEQ(d, co, r1, r2, ci) \
+  ((d) = ((r1) - 1), (co) = (r1) == (r2))   /* should protect from samevar(d,r1/r2) ??? */
+#endif
+
 /* Unary operations.  */
 #ifndef PERFORM_CLZ
 #define PERFORM_CLZ(d, co, r1, ci) \
